@@ -1,7 +1,8 @@
 # coding=utf-8
 import tensorflow as tf
-from tf_geometric.sparse.sparse_matrix import SparseMatrix
-from tf_geometric.sparse.sparse_ops import sparse_diag_matmul, diag_sparse_matmul
+from tf_geometric.sparse.sparse_adj import SparseAdj
+import tf_sparse as tfs
+
 
 # new API
 CACHE_KEY_GCN_NORMED_ADJ_TEMPLATE = "gcn_normed_adj_{}_{}"
@@ -33,14 +34,19 @@ def gcn_norm_adj(sparse_adj, renorm=True, improved=False, cache: dict = None):
         cache_key = compute_cache_key(renorm, improved)
         cached_data = cache.get(cache_key, None)
         if cached_data is not None:
-            return cached_data
+            # return cached_data
+            return SparseAdj(cached_data[0], cached_data[1], cached_data[2])
+        else:
+            if not tf.executing_eagerly():
+                raise Exception("If you want to use cache inside a tf.function, you should manually build the cache before calling the tf.function")
+
 
     fill_weight = 2.0 if improved else 1.0
 
     if renorm:
         sparse_adj = sparse_adj.add_self_loop(fill_weight=fill_weight)
 
-    deg = sparse_adj.reduce_sum(axis=-1)
+    deg = sparse_adj.segment_sum(axis=-1)
     deg_inv_sqrt = tf.pow(deg, -0.5)
     deg_inv_sqrt = tf.where(
         tf.math.logical_or(tf.math.is_inf(deg_inv_sqrt), tf.math.is_nan(deg_inv_sqrt)),
@@ -49,13 +55,15 @@ def gcn_norm_adj(sparse_adj, renorm=True, improved=False, cache: dict = None):
     )
 
     # (D^(-1/2)A)D^(-1/2)
-    normed_sparse_adj = sparse_diag_matmul(diag_sparse_matmul(deg_inv_sqrt, sparse_adj), deg_inv_sqrt)
+    normed_sparse_adj = tfs.sparse_diag_matmul(tfs.diag_sparse_matmul(deg_inv_sqrt, sparse_adj), deg_inv_sqrt)
 
     if not renorm:
         normed_sparse_adj = normed_sparse_adj.add_self_loop(fill_weight=fill_weight)
 
     if cache is not None:
-        cache[cache_key] = normed_sparse_adj
+        # cache[cache_key] = normed_sparse_adj
+        # tf.function will convert numpy arrays as constants, while tensors may be converted into placeholders
+        cache[cache_key] = normed_sparse_adj.index.numpy(), normed_sparse_adj.value.numpy(), normed_sparse_adj._shape.numpy()
 
     return normed_sparse_adj
 
@@ -74,7 +82,8 @@ def gcn_build_cache_for_graph(graph, renorm=True, improved=False, override=False
     if override:
         cache_key = compute_cache_key(renorm, improved)
         graph.cache[cache_key] = None
-    sparse_adj = SparseMatrix(graph.edge_index, graph.edge_weight, [graph.num_nodes, graph.num_nodes])
+
+    sparse_adj = SparseAdj(graph.edge_index, graph.edge_weight, [graph.num_nodes, graph.num_nodes])
     gcn_norm_adj(sparse_adj, renorm, improved, graph.cache)
 
 
@@ -94,7 +103,7 @@ def gcn_norm_edge(edge_index, num_nodes, edge_weight=None, renorm=True, improved
     .. deprecated:: 0.0.56
         Use ``gcn_norm_adj`` instead.
     """
-    sparse_adj = SparseMatrix(edge_index, edge_weight, [num_nodes, num_nodes])
+    sparse_adj = SparseAdj(edge_index, edge_weight, [num_nodes, num_nodes])
     normed_sparse_adj = gcn_norm_adj(sparse_adj, renorm=renorm, improved=improved, cache=cache)
     return normed_sparse_adj.edge_index, normed_sparse_adj.edge_weight
 
@@ -152,9 +161,9 @@ def gcn(x, edge_index, edge_weight, kernel, bias=None, activation=None,
     :return: Updated node features (x), shape: [num_nodes, num_output_features]
     """
 
-    num_nodes = tf.shape(x)[0]
+    num_nodes = tfs.shape(x)[0]
 
-    sparse_adj = SparseMatrix(edge_index, edge_weight, [num_nodes, num_nodes])
+    sparse_adj = SparseAdj(edge_index, edge_weight, [num_nodes, num_nodes])
     normed_sparse_adj = gcn_norm_adj(sparse_adj, renorm, improved, cache)
 
     # SparseTensor is usually used for one-hot node features (For example, feature-less nodes.)
